module sema.LoadModule;

import sema.Visitor,
       sema.AstAction,
       sema.ScopeBuilder,
       sema.ScopeCheck,
       sema.TypeCheck;

import lexer.Lexer;
import parser.Parser;

import basic.SourceManager,
       basic.Message;

import tango.io.Stdout,
       tango.io.FilePath;


class LoadModule : Visitor!(Module[], void)
{
    Module[] visit(Module m, SourceManager sc, MessageHandler messages)
    {
        this.messages = messages;
        this.sc = sc;

        visitModule(m);

        return res;
    }

    void visitImportDecl(ImportDecl i)
    {
        char[] dir;

        foreach(p ; i.packages)
            dir ~= p.get ~ "/";

        dir ~= i.name.name ~".d";

        if(!FilePath(dir).exists)
            messages.report(CannotFindModule, i.name.loc)
                .arg(dir)
                .fatal(ExitLevel.Semantic);

        auto start = sc.addFile(dir);
        auto lexer = new Lexer(start, sc, messages);
        auto parser = new Parser(messages);
        auto action = new AstAction(sc);
        auto m = cast(Module)parser.parse(sc, lexer, action);
        messages.checkErrors(ExitLevel.Parser);

        Module[] mods = (new LoadModule).visit(m, sc, messages);
        (new ScopeBuilder).visit(m);
        (new ScopeCheck).visit(m);
        (new TypeCheck).visit(m);

        res ~= m;
    }

    MessageHandler messages;
    Module[] res;
    SourceManager sc;
}
